@@ -1,16 +1,12 @@
-# -*- coding: utf-8 -*-
-# This module is part of GitPython and is released under
-# the BSD License: http://www.opensource.org/licenses/bsd-license.php
+# This module is part of GitPython and is released under the
+# 3-Clause BSD License: https://opensource.org/license/bsd-3-clause/
 
 from pathlib import Path
 import re
 
 import git
 
-from .lib import (
-    TestBase,
-    with_rw_directory,
-)
+from test.lib import TestBase, with_rw_directory
 
 
 class TestClone(TestBase):
@@ -20,8 +16,8 @@ class TestClone(TestBase):
         garbage_file = non_empty_dir / "not-empty"
         garbage_file.write_text("Garbage!")
 
-        # Verify that cloning into the non-empty dir fails while complaining about
-        # the target directory not being empty/non-existent
+        # Verify that cloning into the non-empty dir fails while complaining about the
+        # target directory not being empty/non-existent.
         try:
             self.rorepo.clone(non_empty_dir)
         except git.GitCommandError as exc:
